"""Add tables to store conversation tags.

Reason:
This migration adds new tables, "conversation_tag" and "conversation_to_tag_mapping"
which allows Rasa X to store arbitrary colored tags for conversations.

Revision ID: 22e9e6ea593c
Revises: 443af39e50e3

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = "22e9e6ea593c"
down_revision = "443af39e50e3"
branch_labels = None
depends_on = None


CONVERSATION_TAG = "conversation_tag"
TAG_VALUE_INDEX = "ct_tag_value_index"

CONVERSATION_TO_TAG_MAPPING = "conversation_to_tag_mapping"
TAG_ID_INDEX = "cttm_tag_id_index"
CONVERSATION_ID_INDEX = "cttm_conversation_id_index"


def upgrade():
    op.create_table(
        CONVERSATION_TAG,
        sa.Column("id", sa.Integer(), nullable=False),
        sa.Column("value", sa.String(255), nullable=False, unique=True),
        # Limited to "6" here since it's a HEX color code
        sa.Column("color", sa.String(6), nullable=False),
        sa.PrimaryKeyConstraint("id"),
    )

    with op.batch_alter_table(CONVERSATION_TAG) as batch_op:
        batch_op.create_index(TAG_VALUE_INDEX, ["value"])

    op.create_table(
        CONVERSATION_TO_TAG_MAPPING,
        sa.Column("tag_id", sa.Integer(), nullable=False),
        sa.Column("conversation_id", sa.String(255), nullable=False),
        sa.ForeignKeyConstraint(["tag_id"], [f"{CONVERSATION_TAG}.id"]),
        sa.ForeignKeyConstraint(["conversation_id"], ["conversation.sender_id"]),
        sa.UniqueConstraint(
            "tag_id", "conversation_id", name="cttm_unique_tag_conversation"
        ),
    )

    with op.batch_alter_table(CONVERSATION_TO_TAG_MAPPING) as batch_op:
        batch_op.create_index(TAG_ID_INDEX, ["tag_id"])
        batch_op.create_index(CONVERSATION_ID_INDEX, ["conversation_id"])


def downgrade():
    with op.batch_alter_table(CONVERSATION_TAG) as batch_op:
        batch_op.drop_index(TAG_VALUE_INDEX)
    op.drop_table(CONVERSATION_TAG)

    with op.batch_alter_table(CONVERSATION_TO_TAG_MAPPING) as batch_op:
        batch_op.drop_index(TAG_ID_INDEX)
        batch_op.drop_index(CONVERSATION_ID_INDEX)
    op.drop_table(CONVERSATION_TO_TAG_MAPPING)
