from http import HTTPStatus
from typing import Text, Dict, Any, NoReturn

from _pytest.monkeypatch import MonkeyPatch
from pytest_sanic.utils import TestClient
from rasax.community.services.event_service import EventService
from tests.unit.api.blueprints.conftest import (
    CONVERSATIONS_ENDPOINT,
    TAGS_ENDPOINT,
    tag_1,
)


def raise_value_error(*_) -> NoReturn:
    raise ValueError


def raise_exception(*_) -> NoReturn:
    raise Exception


async def test_empty_request_fails(
    conversation: Text, test_client: TestClient,
):
    url = f"{CONVERSATIONS_ENDPOINT}/{conversation}/{TAGS_ENDPOINT}"

    response = await test_client.post(url)
    assert response.status == HTTPStatus.BAD_REQUEST


async def test_add_conversation_tag_ok(
    conversation: Text, test_client: TestClient, monkeypatch: MonkeyPatch,
):
    monkeypatch.setattr(EventService, "add_conversation_tags", lambda x, y, z: [])

    url = f"{CONVERSATIONS_ENDPOINT}/{conversation}/{TAGS_ENDPOINT}"

    response = await test_client.post(url, json=tag_1)
    assert response.status == HTTPStatus.OK


async def test_add_conversation_tag_throw(
    conversation: Text, test_client: TestClient, monkeypatch: MonkeyPatch,
):
    monkeypatch.setattr(EventService, "add_conversation_tags", raise_value_error)

    url = f"{CONVERSATIONS_ENDPOINT}/{conversation}/{TAGS_ENDPOINT}"

    response = await test_client.post(url, json=tag_1)
    assert response.status == HTTPStatus.BAD_REQUEST


async def test_delete_conversation_tag_ok(
    conversation: Text,
    test_client: TestClient,
    auth_header: Dict[Text, Text],
    monkeypatch: MonkeyPatch,
):
    monkeypatch.setattr(EventService, "remove_conversation_tag", lambda x, y, z: None)

    url = f"{CONVERSATIONS_ENDPOINT}/{conversation}/{TAGS_ENDPOINT}/100"

    response = await test_client.delete(url, headers=auth_header)
    assert response.status == HTTPStatus.NO_CONTENT


async def test_delete_conversation_tag_throw(
    conversation: Text,
    test_client: TestClient,
    auth_header: Dict[Text, Text],
    monkeypatch: MonkeyPatch,
):
    monkeypatch.setattr(EventService, "remove_conversation_tag", raise_value_error)

    url = f"{CONVERSATIONS_ENDPOINT}/{conversation}/{TAGS_ENDPOINT}/100"

    response = await test_client.delete(url, headers=auth_header)
    assert response.status == HTTPStatus.NOT_FOUND

    monkeypatch.setattr(EventService, "remove_conversation_tag", raise_exception)
    response = await test_client.delete(url, headers=auth_header)
    assert response.status == HTTPStatus.BAD_REQUEST
